package be.cylab.java.wowa.training;

import info.debatty.java.aggregation.WOWA;

import java.util.List;

/**
 * Class of Solution. Solution with distance performance evaluation criterion
 */
public class SolutionDistance extends AbstractSolution {

    /**
     * SolutionDistance constructor. Needs weight number as parameter.
     *
     * @param weights_number
     */
    public SolutionDistance(final int weights_number) {
        super(weights_number);
    }

    /**
     * Constructor solution distance. Made for test to initialize the Random
     * Generator
     *
     * @param weights_number int
     * @param seed           int
     */

    public SolutionDistance(final int weights_number, final int seed) {
        super(weights_number, seed);
    }


    /**
     * Compute the solution fitness score.
     * For each element in the training dataset, compute WOWA and perform
     * a substraction between the WOWA just computed and the expected result
     * from the dataset.
     * The sum of all these differences is the fitness score (distance)
     *
     * @param data
     * @param expected
     */
    final void computeScoreTo(final List<List<Double>> data,
                              final List<Double> expected
    ) {

        this.fitness_score = 0.0;
        for (int i = 0; i < data.size(); i++) {
            List<Double> vector = data.get(i);
            Double target_value = expected.get(i);
            WOWA wowa = new WOWA(
                    Utils.convertListDoubleToArrayDouble(this.weights_w),
                    Utils.convertListDoubleToArrayDouble(this.weights_p));
            Double aggregated_value = wowa.aggregate(
                    Utils.convertListDoubleToArrayDouble(vector));
            this.fitness_score += Math.pow(target_value - aggregated_value, 2);
        }
        this.fitness_score = Math.sqrt(this.fitness_score);

    }

}
